﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using MathNet.Numerics.LinearAlgebra;
using MathNet.Numerics.LinearAlgebra.Double;

public class FlatlandMovement : MonoBehaviour
{
    protected bool toggle = false;
    protected bool grabbed = false;

    protected int cnt = 0;
    protected double beta = 0.5f; // v/c

    protected MatrixBuilder<double> M = Matrix<double>.Build;
    protected VectorBuilder<double> V = Vector<double>.Build;

    public Vector3 alpha = new Vector3(0.1f, 0.0f, 0.0f); // proper acceleration
    public Vector3 v;
    public Vector3 orientation;

    public LevelManager levelManager;
    public double gamma = 1.0f;


    public double time = 0.0f;

    public GameObject theobject;

    // 마우스로 이동할때 하용하는 변수들.
    bool isAutoMove = false;

    Vector3 nowDest;
    /// <summary>
    /// 앞으로 남은 목적지들.
    /// </summary>
    List<Vector3> dests = new List<Vector3>();
    /// <summary>
    /// 앞으로 남은 목적지 속력
    /// </summary>
    List<float> pathVelocitys = new List<float>();

    protected void Update()
    {
        if (isAutoMove)
        {

            Vector3 tmp = new Vector3(transform.position.x,0,transform.position.z);
            if (SpaceLength(nowDest, transform.position) < 0.1f || SpaceInnerPorduct(nowDest - tmp, v) < 0) 
            {
                //목적지 근접 or 넘어가면
                Debug.Log("dest" + nowDest);
                Debug.Log("position" + transform.position);

                if (dests.Count >= 1)
                {
                    // 다음 목적지가 있을떄.
                    MoveTo(dests[0], pathVelocitys[0]); //이동한다.
                    dests.RemoveAt(0);
                    pathVelocitys.RemoveAt(0);

                }
                else
                {
                    //다음목적지 없을때
                    v = Vector3.zero;
                    nowDest = Vector3.zero;
                    isAutoMove = false;
                }

            }
        }
    }
    /// <summary>
    /// 시간축 y을 무시한
    /// 거리 
    /// </summary>
    /// <param name="a"></param>
    /// <param name="b"></param>
    /// <returns></returns>
    protected float SpaceLength(Vector3 a, Vector3 b)
    {
        return Mathf.Sqrt((a.x - b.x) * (a.x - b.x) + (a.z - b.z) * (a.z - b.z));
    }
    /// <summary>
    /// 시간축 y을 무시한
    /// 내적
    /// </summary>
    /// <param name="a">xy가 공간</param>
    /// <param name="b">xz가 공간</param>
    /// <returns></returns>
    protected float SpaceInnerPorduct(Vector3 a, Vector3 b)
    {
        return a.x * b.x + a.z * b.z;
    }

    /// <summary>
    /// 여러 위치를 거쳐서 이동합니다.
    /// </summary>
    /// <param name="path">목적지 리스트 0 번은 현재 위치 x y 가 공간</param>
    /// <param name="v"> 경로별 속력 0번은 0.0f </param>
    public bool MoveTo(List<Vector3> path, List<float> v)
    {
        //Debug.Log("aa");
        for (int i = 1; i < v.Count; i++) 
        {
            //Debug.Log(v[i]);
            //속력 확인
            if (v[1] < 0.0001f)
            {
                return false;
            }
        }

        if (path.Count >= 2)
        {
            dests = new List<Vector3>();
            //xy 공간 -> xz 공간
            foreach(var a in path)
            {
                dests.Add(new Vector3(transform.position.x + a.x, 0, transform.position.z + a.y));
            }

            pathVelocitys = new List<float>(v);
            //더미 제거
            dests.RemoveAt(0);
            pathVelocitys.RemoveAt(0);
        }

        MoveTo(dests[0], pathVelocitys[0]);

        dests.RemoveAt(0);
        pathVelocitys.RemoveAt(0);

        return true;
    }
    /// <summary>
    /// 목적지로 바로 이동합니다.
    /// </summary>
    /// <param name="path">목적지 x z 가 공간</param>
    /// <param name="v">속력 0<v<1</param>
    public bool MoveTo(Vector3 dest, float v)
    {
        dest.y = 0;
        Debug.Log("bb");
        Debug.Log(dest);
        if (v < 0.0001f)
        {
            //v가 0일때.
            return false;
        }
        //속도와 목적지 설정.
        Vector3 tmp = dest-transform.position;
        Debug.Log("move vector: " + tmp);
        tmp.y = 0;
        this.v = tmp.normalized * v * (float)Constants.c;
        Debug.Log("v : "+ this.v);
        this.nowDest = dest;

        isAutoMove = true;
        return true;
    }

}
